@@ -6,7 +6,7 @@ from octopus_python_client.common import config, get_one_type_save, get_single_i
     update_single_item_save, create_single_item_from_local_file, item_type_deployment_processes, \
     clone_single_item_from_remote_item, delete_single_item_by_name_or_id, get_child_item_save, \
     update_child_item_from_local_save, clone_child_item_from_another_parent_save, get_types_save, \
-    item_types_only_ourter_space, item_types_inside_space, deployment_process_id_key, steps_key, \
+    item_types_only_outer_space, item_types_inside_space, deployment_process_id_key, steps_key, \
     merge_single_item_save, delete_one_type, api_key_key, octopus_endpoint_key, octopus_name_key, user_name_key, \
     password_key, double_hyphen, verify_space, get_spaces_save, get_task_status, wait_task
 from octopus_python_client.migration import Migration
@@ -80,10 +80,10 @@ def _parse_args():
                              'regardless whether they are above Spaces; if (not item_types) and octopus_space_id, '
                              'get all item types below octopus_space_id; '
                              'list like "accounts,actiontemplates,artifacts" is also accepted; '
-                             'item types above Spaces: ' + ", ".join(item_types_only_ourter_space) +
+                             'item types above Spaces: ' + ", ".join(item_types_only_outer_space) +
                              "; \nitem types above and under Spaces: " + ", ".join(item_types_inside_space))
     parser.add_argument("-tp", "--item_type", help="one of item types above Spaces: " + ", ".join(
-        item_types_only_ourter_space) + "; \nitem types above and under Spaces: " + ", ".join(item_types_inside_space))
+        item_types_only_outer_space) + "; \nitem types above and under Spaces: " + ", ".join(item_types_inside_space))
     parser.add_argument("-nm", "--item_name", help="item name: e.g. project_name")
     parser.add_argument("-id", "--item_id", help="item id: e.g. Lifecycles-1")
     parser.add_argument("-bn", "--base_item_name", help="base item name, either local or on Octopus server")
@@ -101,14 +101,10 @@ def _parse_args():
     parser.add_argument("-rv", "--release_version", help="release version for creating a new release")
     parser.add_argument("-as", "--add_suffix",
                         help="if present, add suffix to variable sets name")
-    parser.add_argument("-rs", "--remove_suffix", help="if present, remove suffix from variable sets name")                        
+    parser.add_argument("-rs", "--remove_suffix", help="if present, remove suffix from variable sets name")
     parser.add_argument("-pj", "--project_name", help="project name")
     parser.add_argument("-cn", "--channel_name", help="channel name")
     parser.add_argument("-nt", "--notes", help="notes")
-    parser.add_argument("-pv", "--package_version_json",
-                        help="user selected package versions, e.g. \""
-                             r"{\"package.near\": \"20.0225.1714\", \"package.main\": \"20.0325.0839\"}" "\"")
-    parser.add_argument("-vs", "--variable_set_name", help="variable set name")
     parser.add_argument("-ri", "--release_id", help="release id for deployment")
     parser.add_argument("-en", "--environment_name", help="environment name, like Integration")
     parser.add_argument("-tn", "--tenant_name", help="tenant name, like cd-near")
@@ -123,8 +119,8 @@ def run():
 
     if args.octopus_endpoint:
         config.octopus_endpoint = args.octopus_endpoint
-    if not config.octopus_endpoint:
-        raise ValueError("octopus_endpoint must not be empty")
+    if not config.octopus_endpoint or not config.octopus_endpoint.endswith("/api/"):
+        raise ValueError("octopus_endpoint must end with /api/")
 
     if args.octopus_name:
         config.octopus_name = args.octopus_name
@@ -241,8 +237,7 @@ def run():
     elif args.action == Actions.action_create_release:
         ReleaseDeployment.create_release_direct(
             release_version=args.release_version, project_name=args.project_name, channel_name=args.channel_name,
-            notes=args.notes, space_id_name=space_id, package_version_json=args.package_version_json,
-            packages_variable_set_name=args.variable_set_name)
+            notes=args.notes, space_id_name=space_id)
     elif args.action == Actions.action_create_deployment:
         ReleaseDeployment.create_deployment_direct(
             release_id=args.release_id, environment_name=args.environment_name, tenant_name=args.tenant_name,
@@ -250,8 +245,7 @@ def run():
     elif args.action == Actions.action_create_release_deployment:
         ReleaseDeployment.create_release_deployment(
             release_version=args.release_version, project_name=args.project_name, channel_name=args.channel_name,
-            notes=args.notes, space_id_name=space_id, package_version_json=args.package_version_json,
-            packages_variable_set_name=args.variable_set_name, environment_name=args.environment_name,
+            notes=args.notes, space_id_name=space_id, environment_name=args.environment_name,
             tenant_name=args.tenant_name, comments=args.comments)
 
     else:
